local T = ...
local _
local num_scanned = 0
local est_total = 0
local scan_count = 0

function init_scan_data()
	scan_count = 0
	_, T.Scan.num_scanned, T.Scan.est_total = T.RoidsScanned()
end

function T.Events:UPDATE_SCAN_STATS()
	init_scan_data()
end

function T.Events:ENTERED_STATION()
	T.Scan.AutoScanStartStop(false)
	T.SaveData()
end

function T.Events:LEAVING_STATION()
	T.Scan.GetScannerRange()
end

function T.Events:MSG_LOGOFF_TIMER(_, msg)
	if (msg == "10 seconds to logoff") then
		T.SaveData()
	end
end

function T.Events:PLAYER_ENTERED_GAME()
	T.CurrentSystemID = GetCurrentSystemid()
	T.UI.navmap:SetSystemID(T.CurrentSystemID)
	T.UI.navmap:SetSectorID(GetCurrentSectorid())
	self:LEAVING_STATION()
	T.LoadData()
	init_scan_data()

	if (T.IsMobile() and not T.use_mobile_layout) then
		local Yres = gkinterface.GetYResolution()
		if (T.UI.targetbtn.y + T.UI.targetbtn.h > Yres) then
			local msgdlg = msg2dlgtemplate1(
				"MDI-RoidDB",
				"PC Layout does not fit on your screen.  Reverting to Mobile Layout.",
				"OK",
				function(self)
					T.use_mobile_layout = true
					T.SaveSettings()
					ReloadInterface()
				end,
				nil,
				nil,
				""
			)
			T.Save.timer:SetTimeout(1, function() ShowDialog(msgdlg) end)
		end
	end
end

function T.Events:PLAYER_GOT_HIT()
	if (not T.disable_auto_close) then
		HideDialog(T.UI.maindlg)
	end
end

function T.Events:PLAYER_LOGGED_OUT()
	T.Scan.AutoScanStartStop(false)
end

function T.Events:PLUGINS_LOADED()
end

function T.Events:rHUDxscale()
	if (IsConnected()) then
		T.Font = Font.Default * T.FONT_SCALE
		iup.Destroy(T.UI.maindlg)
		T.UI.BuildUI()
	end
end

function T.Events:SECTOR_CHANGED(_, sectorid)
	T.Scan.AutoScanStartStop(false)
	T.SaveData(T.CurrentSystemID) -- Attempt to save data. Can't use GetCurrentSystemid() in case they went through a wormhole.
	T.CurrentSystemID = GetSystemID(sectorid)
	-- update navmap ids
	T.UI.navmap:SetSystemID(T.CurrentSystemID)
	T.UI.navmap:SetSectorID(sectorid)
	T.CurrentSectorNodeID = nil
	T.Scan.roid_list = {} -- clear the autoscan roid list

	init_scan_data()
end

function T.Events:TARGET_CHANGED(_, data)
	if (T.Scan.Lock()) then return end


	-- Kill the ping timer before any other action, in case another action restarts it
	T.Scan.ping_timer:Kill()
	T.Scan.timer:Kill()

	if (T.Scan.ningAuto) then
		local sector_data = T.GetData(GetCurrentSystemid(), GetCurrentSectorid())
		local _, roidID = radar.GetRadarSelectionID()
		local already_scanned = sector_data[roidID]
		if (not roidID or already_scanned) then
			if (already_scanned) then
				HUD:PrintSecondaryMsg("Asteroid "..roidID.." has already been scanned.  New target acquired.")
			end
			T.Scan.LockFunction(T.Scan.TargetNextClosestRoid)
		elseif (not T.IsAsteroid()) then
			print("\127FFFFFFManual target obtained or target lost.\127o")
			T.Scan.AutoScanStartStop()
		end
	end
end

function T.Events:TARGET_SCANNED(_, minerals, nodeID, roidID)
	if (minerals == "Object too far to scan\n") then
		-- do nothing
	elseif (minerals:match("Minerals:")) then
		local sectorID = GetCurrentSectorid()
		local systemID = GetCurrentSystemid()
		local sector_data = T.GetData(systemID, sectorID, true)

		-- flag sector pending update
		-- summary will update when UI opens

		if (not sector_data[roidID] or T.Scan.re) then
			-- Don't store roid data if auto-store is disabled and the modifier key isn't down. /+roiddbstore
			if (T.Scan.dont_store and not T.Scan.store_ok) then return end

			sector_data[roidID] = {id = roidID, ore = {}}
			-- operate on the minerals string to create roid data
			minerals:gsub("(%w+) Ore: ([%d.]+)%%", function(ore_type, ore_pct)
				ore_pct = tonumber(ore_pct)
				ore_type = T.ore_reference[3][ore_type]
				if (sector_data[roidID].ore[ore_type] ~= ore_pct) then
					sector_data[roidID].ore[ore_type] = ore_pct
				end
			end)
			T.pending:AddRoid(sectorID, roidID, sector_data[roidID])

			if (T.Scan.dont_store and T.Scan.store_ok) then
				HUD:PrintSecondaryMsg("Asteroid "..roidID.." data stored.")
				T.Scan.store_ok = false
			end

			-- Print a message showing the current state of scanned roids in the sector.
			T.Scan.num_scanned = T.Scan.num_scanned + 1
			if (roidID > T.Scan.est_total) then T.Scan.est_total = roidID end
			HUD:PrintSecondaryMsg(T.Scan.num_scanned.." of "..T.Scan.est_total.." scanned.")
		end

		T.Scan.ping_timer:Kill()


		if (T.Scan.ningAuto) then -- and not T.Scan.timer:IsActive()) then
			T.Scan.LockFunction(T.Scan.TargetNextClosestRoid)
		end
	end
end

function T.Events:UNLOAD_INTERFACE()
	T.SaveData()
	UnregisterUserCommand("roiddb", function() ShowDialog(T.UI.maindlg) end)
	UnregisterUserCommand("roiddbscan", T.Scan.AutoScanStartStop)
end

RegisterEvent(T.Events, "ENTERED_STATION")
RegisterEvent(T.Events, "LEAVING_STATION")
RegisterEvent(T.Events, "MSG_LOGOFF_TIMER")
RegisterEvent(T.Events, "PLAYER_ENTERED_GAME")
RegisterEvent(T.Events, "PLAYER_GOT_HIT")
RegisterEvent(T.Events, "PLAYER_LOGGED_OUT")
RegisterEvent(T.Events, "PLUGINS_LOADED")
--RegisterEvent(T.Events, "rHUDxscale")
RegisterEvent(T.Events, "SECTOR_CHANGED")
RegisterEvent(T.Events, "TARGET_CHANGED")
RegisterEvent(T.Events, "TARGET_SCANNED")
RegisterEvent(T.Events, "UNLOAD_INTERFACE")
RegisterEvent(T.Events, "UPDATE_SCAN_STATS")
