local T = ...

T.Tab = {
	title = "Asteroids",
	columns = {
		{
			title = "ID",
			width = 0.125,
		},
		{
			title = "Ore",
			width = 0.675,
		},
		{
			title = "Distance",
			width = 0.2,
		},
	},
	filters = {
		{
			title = "Ore",
		},
		{
			title = "Aq",
			pages = {},
		},
		{
			title = "Unscanned",
		},
	},
	-- -------------------
	current_target_list_index,
	current_active_roidID,
	current_select_roidID,
	filter_index = -1,
	filter_ore_type,
	filter_page = {0, 0},
	is_active = false,
	roid_list = {},
	Scan = {
		full_freq = 15,
		sub_freq = 5,
		sub_max = 100,
		subsub_max = 25,
		cycle_count = 15,  -- force full_scan on first run.
		interval = 1000,
		full,
		timer = Timer(),
	},
}
-- populate the 2nd filter pages
for i, o in ipairs(T.ore_reference[1]) do
	T.Tab.filters[2].pages[i] ={title = o}
end

function T.Tab:ChangeTarget(oldID, newID, select_only)
	local roid_list = T.Tab.roid_list

	if (roid_list[newID]) then

		if (select_only) then
			-- Set the "select" highlight if the newID is not equal to the current target ID
			if (newID == targetlist.API.GetActiveItemIndex(self)) then
				-- don't change the highlight if the new selector index equals the active index
			else
				roid_list[newID].highlight = "select"
			end

			-- Clear the old highlight, so long as the oldID isn't the current target ID
			if (oldID == targetlist.API.GetActiveItemIndex(self)) then
				-- don't change the highlight if the old selector index equals the active index
			elseif (oldID and oldID ~= newID) then
				roid_list[oldID].highlight = nil
			end

		else -- active item change

			roid_list[newID].highlight = true

			T.Scan.LockFunction(function() radar.SetRadarSelection(T.CurrentSectorNodeID, roid_list[newID][1]) end)
			self.current_target_roidID = nil

			if (oldID and oldID ~= newID) then
				roid_list[oldID].highlight = nil
			end

		end

	else -- there is no roid in the list associated with the newID
		if (oldID and oldID ~= newID) then
			roid_list[oldID].highlight = nil
		end
	end
	targetlist.API.UpdateDisplay(self, roid_list)
end

function T.Tab:ChangeFilter(index, page)
	self.filter_index = index

	if (index == 2) then
		local r, num_filter_pages = pcall(function() return #self.filters[index].pages end)
		local num_filter_pages = (r and num_filter_pages or 1)
		self.filter_ore_type = self.filters[index].pages[page].title
		self.filters[index].title = self.filter_ore_type
		--targetlist.API.UpdateFilter(self.filter_ore_type)
	else
		self.filter_ore_type = nil
	end

	self.Scan.cycle_count = self.Scan.full_freq -- force full scan on start
	self:HUD_SHOW(nil, "ship")
end

function T.Tab:PinTarget(index)
	local listID = index or targetlist.API.GetActiveItemIndex(self)
	local roidtbl = self.roid_list[listID]
	if (roidtbl) then
		local roidID = roidtbl[1]
		roidtbl.pin = T.TogglePin(GetCurrentSectorid(), roidID)
		self.current_target_roidID = roidID
	end
end

function T.Tab:SECTOR_CHANGED(_, sectorID)
	self.current_select_roidID = nil
end

function T.Tab:TARGETLIST_TAB_CHANGED()
 	self.is_active = targetlist.API.IsActiveTab(self)
	if (self.is_active) then
		self.current_active_roidID = ({radar.GetRadarSelectionID()})[2]
		if (not targetlist.API.GetActiveItemIndex(self)) then
			T.current_select_roidID = nil
		end
		RegisterEvent(self, "TARGET_CHANGED")
		RegisterEvent(self, "WEAPON_PROGRESS_START")
		RegisterEvent(self, "WEAPON_PROGRESS_STOP")
	else
		UnregisterEvent(self, "TARGET_CHANGED")
		UnregisterEvent(self, "WEAPON_PROGRESS_START")
		UnregisterEvent(self, "WEAPON_PROGRESS_STOP")
		self.Scan.timer:Kill()
	end
end

function T.Tab:TARGET_CHANGED(_, data)
	if (T.Scan.Lock()) then return end

	-- The display timer may be inactive if there are initially no asteroids in range when the tab is activated
	-- If inactive, then check if the new target is a roid before restarting it.
	if (not self.Scan.timer:IsActive() and T.IsAsteroid()) then
		self:HUD_SHOW(nil, "ship")

	else
		-- If the new target matches a list item, then cause the display to be updated (highlight)
		local new_target_nodeID, new_target_objectID = radar.GetRadarSelectionID()
		local new_target_listindex

		for i, roidtbl in ipairs(self.roid_list) do
			if (new_target_nodeID == T.Scan.GetSectorNodeID() and roidtbl[1] == new_target_objectID) then
				new_target_listindex = i
				break
			end
		end
		self:ChangeTarget(targetlist.API.GetActiveItemIndex(self), new_target_listindex)
	end
end

-- HUD_SHOW is a more consistent event to trigger the start of the scan with.
function T.Tab:HUD_SHOW(_, hud)
	if(self.is_active and hud == "ship") then
		self.roid_list = {}
		self.Scan.cycle_count = self.Scan.full_freq
		if (targetlist.API.GetFilterIndex(self) == 3) then
			T.Scan.LockFunction(T.Scan.UnscannedRoids)
		else
			T.Scan.LockFunction(T.Scan.ScannedRoids)
		end
	end
end

function T.Tab:HUD_HIDE(_, event, data)
	if (self.is_active) then
		self.Scan.timer:Kill()
	end
end

function T.Tab:PLAYER_ENTERED_GAME()
	--self:TARGETLIST_TAB_CHANGED()
	if (not (isdeclared("targetlist") and targetlist.API)) then
		print("\127FFFFFFThe MDI-RoidDB Asteroids Tab requires MDI-TargetList v4.2+.\127o")
	end
end

function T.Tab:PLAYER_LOGGED_OUT()
	self.Scan.timer:Kill()
end

function T.Tab:PLUGINS_LOADED()
	if (isdeclared("targetlist") and targetlist.API) then
		if (targetlist.API.Version >= 3) then
			targetlist.API.RegisterTab(T.Tab)
		else
			print("\127FFFFFFThe MDI-TargetList API version is different than expected.  MDI-RoidDB Tab was not loaded.\127o")
		end
	end
end

function T.Tab:WEAPON_PROGRESS_START()
	self.Scan.timer:Kill()
end

function T.Tab:WEAPON_PROGRESS_STOP()
	self.Scan.timer:SetTimeout(self.Scan.interval, function() self:HUD_SHOW(nil, "ship") end)
end


RegisterEvent(T.Tab, "PLUGINS_LOADED")
RegisterEvent(T.Tab, "PLAYER_ENTERED_GAME")
RegisterEvent(T.Tab, "PLAYER_LOGGED_OUT")
RegisterEvent(T.Tab, "SECTOR_CHANGED")
RegisterEvent(T.Tab, "TARGETLIST_TAB_CHANGED")
RegisterEvent(T.Tab, "HUD_HIDE")
RegisterEvent(T.Tab, "HUD_SHOW")
--RegisterEvent(T.Tab, "TARGET_CHANGED") -- Only registered when the tab is active
--RegisterEvent(T.Tab, "WEAPON_PROGRESS_START")  -- Only registered when the tab is active
--RegisterEvent(T.Tab, "WEAPON_PROGRESS_STOP")  -- Only registered when the tab is active


local function ProcessUnscannedRoids(sector_data, roid_list, id, orig_targetID, full_scan)
	local roidID = (full_scan and id or roid_list[id][1]) -- "id" could be either the roidID or the listID.
	radar.SetRadarSelection(T.Scan.GetSectorNodeID(), roidID) -- don't ScanLock here.  It's already inside of a ScanLock

	-- Necessary to make sure we actually changed roids.  Won't change if not in range.
	local _, new_roidID = radar.GetRadarSelectionID()
	if (new_roidID == roidID and T.IsAsteroid()) then
		if (not sector_data[roidID]) then
			-- If the current targeted roid has not yet been scanned
			local distance = math.floor(GetTargetDistance() or 9999)
			local ore_info = ""

			if (full_scan) then
				table.insert(roid_list, {roidID, "Asteroid #"..roidID, distance, highlight = roidID == orig_targetID})
			else
				roid_list[id][3] = distance
				roid_list[id]["highlight"] = roidID == orig_targetID
			end

		else --if (id ~= roidID) then
			table.remove(roid_list, id)
		end
	end
end

function T.Scan.UnscannedRoids()
	-- don't scan if an auto-scan is in progress
	if (T.Scan.ningAuto) then
		T.Tab.roid_list = {{-1,"Auto-Scan in progress"}}
		T.Tab.Scan.cycle_count = T.Tab.Scan.full_freq

	elseif (T.Scan.GetSectorNodeID()) then -- don't ScanLock here.  It's already inside of a ScanLock
		local sector_data = T.GetData(GetCurrentSystemid(), GetCurrentSectorid())

		local orig_nodeID, orig_targetID = radar.GetRadarSelectionID()
		gkinterface.GKProcessCommand("RadarNone")

		-- ----------------------------------------------------
		local cur_count, max_count = 0, 0
		local full_scan = false

		if (T.Tab.Scan.cycle_count >= T.Tab.Scan.full_freq) then
			-- Full Scan
			full_scan = true
			T.Tab.Scan.cycle_count = 0
			T.Tab.roid_list = {}

		elseif ((T.Tab.Scan.cycle_count % T.Tab.Scan.sub_freq) == 0) then
			-- Sub Scan
			max_count = #T.Tab.roid_list <= T.Tab.Scan.sub_max and #T.Tab.roid_list or T.Tab.Scan.sub_max

		else
			-- Sub Sub Scan
			max_count = #T.Tab.roid_list <= T.Tab.Scan.subsub_max and #T.Tab.roid_list or T.Tab.Scan.subsub_max
		end
		-- ----------------------------------------------------

		local roid_list = T.Tab.roid_list

		if (full_scan) then
			for roidID = 1, 20000 do
				ProcessUnscannedRoids(sector_data, roid_list, roidID, orig_targetID, full_scan)
			end

		else
			for listID, roidtbl in ipairs(roid_list) do
				ProcessUnscannedRoids(sector_data, roid_list, listID, orig_targetID)
				cur_count = cur_count + 1
				if (cur_count == max_count) then break end
			end
		end

		table.sort(roid_list, function(a, b)
			return a[3] < b[3] -- distance index
		end)

		-- Retarget our original target
		if (orig_nodeID) then
			radar.SetRadarSelection(orig_nodeID, orig_targetID)
		else
			gkinterface.GKProcessCommand("RadarNone")
		end

	end

	targetlist.API.UpdateDisplay(T.Tab, T.Tab.roid_list)
	T.Tab.Scan.cycle_count = T.Tab.Scan.cycle_count + 1
	T.Tab.Scan.timer:SetTimeout(T.Tab.Scan.interval, function() T.Scan.LockFunction(T.Scan.UnscannedRoids) end)
end

local function filtered_roid_sort(a, b)
	if (T.Tab.filter_ore_type) then
		-- Sort filtered roids first by ore concentration and then by distance
		local A = tonumber(a[2]:match(T.Tab.filter_ore_type..":([%w.]+)")) or 0
		local B = tonumber(b[2]:match(T.Tab.filter_ore_type..":([%w.]+)")) or 0
		return A > B or (A == B and a[3] < b[3]) or false -- Default: False covers the condition when A < B
	else
		-- Sort unfiltered roids by distance
		return a[3] < b[3]
	end
end

function T.Scan.ScannedRoids()
	-- don't scan if an Auto-scan is in progress
	local sector_nodeID = T.Scan.GetSectorNodeID()

	if (T.Scan.ningAuto) then
		T.Tab.roid_list = {{"","Auto-Scan in progress"}}
		T.Tab.Scan.cycle_count = T.Tab.Scan.full_freq

	-- only try to scan if we've got a nodeid for the sector
	elseif (sector_nodeID) then -- don't ScanLock here.  It's already inside of a ScanLock
		local sectorID = GetCurrentSectorid()
		-- Save the original target
		local orig_nodeID, orig_targetID = radar.GetRadarSelectionID()
		gkinterface.GKProcessCommand("RadarNone")

		-- ---------------------------------------------------------
		-- Determine the level of scan/update being performed (full, sub, subsub)
		local cur_count, max_count = 0, 0
		local scan_data = T.Tab.roid_list -- Set scan_data to the current roid_list.  A full scan will change this to the sector data for reference purposes
		local full_scan = false
		if (T.Tab.Scan.cycle_count >= T.Tab.Scan.full_freq) then
			-- Full Scan
			full_scan = true
			T.Tab.Scan.cycle_count = 0

			-- Reference the sector data for a list of scanned roids to check for.
			local sector_data = T.GetData(GetCurrentSystemid(), sectorID)
			scan_data = sector_data

			-- Get the roid IDs for the currently selected items.  This is only necessary on a full scan.
			-- Otherwise there is no way to reference the roid ID later.
			T.Tab.current_active_roidID = orig_nodeID == sector_nodeID and T.Tab.roid_list[targetlist.API.GetActiveItemIndex(T.Tab)]
			T.Tab.current_active_roidID = T.Tab.current_active_roidID and T.Tab.current_active_roidID[1]

			T.Tab.current_select_roidID = T.Tab.roid_list[targetlist.API.GetSelectItemIndex(T.Tab)]
			T.Tab.current_select_roidID = T.Tab.current_select_roidID and T.Tab.current_select_roidID[1]

			-- Clear the roid_list in case some roids have moved out of range
			T.Tab.roid_list = {}

		elseif ((T.Tab.Scan.cycle_count % T.Tab.Scan.sub_freq) == 0) then
			-- Sub Scan
			max_count = #T.Tab.roid_list <= T.Tab.Scan.sub_max and #T.Tab.roid_list or T.Tab.Scan.sub_max

		else
			-- Sub Sub Scan
			max_count = #T.Tab.roid_list <= T.Tab.Scan.subsub_max and #T.Tab.roid_list or T.Tab.Scan.subsub_max
		end

		-- ---------------------------------------------------------
		-- cycle through scanned roids.  populate the roid_list with all roids in range or update the distances of those already scanned
		-- Not using an iterator so that we can table.remove() items from roid_list if necessary
		for i = table.maxn(scan_data), 1, -1 do
			local roidID = i
			local roidtbl = scan_data[roidID]

			-- If this isn't a full scan, then the roidID is going to be the list index.  This needs to be fixed by referencing the actual roidID from the roidtbl
			roidID = full_scan and roidID or roidtbl[1]
			-- Target the roid in order to get it's information
			radar.SetRadarSelection(sector_nodeID, roidID) -- don't ScanLock here.  It's already inside of a ScanLock

			-- Necessary to make sure we actually changed roids.  Won't change if not in range.
			local new_nodeID, new_roidID = radar.GetRadarSelectionID()

			if (new_roidID == roidID) then
				local distance = math.floor(GetTargetDistance() or 9999) -- Occasionally won't return a distance.  Set to 9999 as a default
				local ore_info = ""
				local insert_into = not full_scan

				-- Populate the ore info string for display.  roidtbl is the roid table from the sector data
				if (full_scan and roidtbl) then -- because we're not using an iterator, roidtbl could be nil.
					for _, ore_type in ipairs(T.ore_reference[1]) do
						local ore_pct = roidtbl.ore[ore_type]
						if (ore_pct) then
							local ore_matches_filter = ore_type == T.Tab.filter_ore_type
							insert_into = insert_into or (ore_matches_filter or targetlist.API.GetFilterIndex(T.Tab) == 1)
							ore_info = ore_info..(ore_matches_filter and "\127FF0000" or "")..ore_type..":"..ore_pct.."\127o "
						end
					end
				end

				-- Insert into or Update the roid list.
				if (insert_into) then
					if (full_scan) then
						local new_roid_list_item = {
							roidID,
							ore_info,
							distance,
							pin = T.IsPinned(sectorID, roidID)
						}
						-- Active highlight takes priority over select highlight
						if ((roidID == T.Tab.current_active_roidID or roidID == orig_targetID)) then
							new_roid_list_item["highlight"] = true

						elseif (roidID == T.Tab.current_select_roidID) then
							new_roid_list_item["highlight"] = "select"
						end

						table.insert(T.Tab.roid_list, new_roid_list_item)
					else
						roidtbl[3] = distance
					end

				end
				cur_count = cur_count + 1

			elseif (not full_scan) then
				-- The roid went out of range.  Remove it from the list and force a full scan on the next interval
				table.remove(T.Tab.roid_list, i) -- Explicitly using the roid_list table rather than scan_data.  I'm being paranoid about accidentally deleting data.
				T.Tab.Scan.cycle_count = T.Tab.Scan.full_freq
			end

			if (not full_scan and cur_count == max_count) then break end
		end

		table.sort(T.Tab.roid_list, function(a, b)
			local A, B = T.IsPinned(sectorID, a[1]), T.IsPinned(sectorID, b[1])

			-- Sort pinned roids first.
			if (A and B) then
				return filtered_roid_sort(a, b)
			elseif (A or B) then
				-- If only one roid is pinned then sort it first
				return A or not B
			end

			-- Sorting unpinned roids
			return filtered_roid_sort(a, b)
		end)

		-- Retarget our original target
		if (orig_nodeID) then
			radar.SetRadarSelection(orig_nodeID, orig_targetID)
		else
			gkinterface.GKProcessCommand("RadarNone")
		end

		if (#T.Tab.roid_list == 0) then
			T.Tab.roid_list = {{"","No scanned asteroids"}, {"", "in range."}}
		end

	else
		T.Tab.roid_list = {{"","No asteroids detected."}}
		targetlist.API.UpdateDisplay(T.Tab, T.Tab.roid_list)
		return
	end

	targetlist.API.UpdateDisplay(T.Tab, T.Tab.roid_list)
	T.Tab.Scan.cycle_count = T.Tab.Scan.cycle_count + 1

	T.Tab.Scan.timer:SetTimeout(T.Tab.Scan.interval, function() T.Scan.LockFunction(T.Scan.ScannedRoids) end)
end

